package org.jastadd.relast.compiler;
import org.jastadd.relast.ast.*;


import static org.jastadd.relast.compiler.Utils.filterToList;

import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.regex.Pattern;
import org.jastadd.relast.compiler.options.CommandLine;
import org.jastadd.relast.compiler.options.EnumOption;
import org.jastadd.relast.compiler.options.FlagOption;
import org.jastadd.relast.compiler.options.Option;
import org.jastadd.relast.compiler.options.StringOption;
import org.jastadd.relast.compiler.options.CommandLine.CommandLineException;

import beaver.Parser;

public class Compiler {
	protected ArrayList<Option<?>> options;
	protected FlagOption optionPrintDot;
	protected FlagOption optionFlatPrettyPrint;
	protected CommandLine commandLine;

	public Compiler(String args[]) throws CommandLineException {
		options = new ArrayList<>();
		addOptions();

		commandLine = new CommandLine(options);
		commandLine.parse(args);

		Program p = parseProgram(commandLine.getArguments().get(0));

		if (!p.errors().isEmpty()) {
			System.err.println("Errors:");
			for (ErrorMessage e: p.errors()) {
				System.err.println(e);
			}
			System.exit(1);
		} else {
			System.out.println(p.generateAbstractGrammar());
		}
/*		} else {
			if (optionPrintDot.isSet()) {
				System.out.println(p.genDot());
			} else if (optionFlatPrettyPrint.isSet()) {
				System.out.println(p.flatPrettyPrint());
			} else {
				System.out.println("# AST");
				System.out.println(p.dumpTree());

				FlatState s = p.stoppingState();
				for (FlatDependency d: s.getDependencyList()) {
					System.out.println(d.sourceDecl());
					System.out.println(d.sourceDecl().getAccess() + " " + d.targetDecl().getAccess());
				}
				for (Node n: s.getNodes()) {
					System.out.println(n + ", succ: " + n.succ());
				}
				System.out.println(s.startingNodes());
				System.out.println(s.endingNodes());
				System.out.println("# Transitive reduction");
				System.out.println(p.startingState().transitiveReduction());
			}
		}*/
	}


	protected void addOptions() {
		optionPrintDot = addOption(new FlagOption("dot", "print DOT format"));
		optionFlatPrettyPrint = addOption(new FlagOption("flat", "flat pretty print"));
	}

	protected <OptionType extends Option<?>> OptionType addOption(OptionType option) {
		options.add(option);
		return option;
	}

	private Program parseProgram(String file) {
		FileReader reader = null;
		try {
			reader = new FileReader(file);
		} catch (FileNotFoundException e) {
			System.err.println(e.getMessage());
			System.exit(1);
		}
		return parse(reader, file);
	}

	private Program parse(Reader reader, String file) {
		RelAstScanner scanner = new RelAstScanner(reader);
		RelAstParser parser = new RelAstParser();

		try {
			return (Program) parser.parse(scanner);
		} catch (IOException e) {
			System.err.println(e.getMessage());
			System.exit(1);
		} catch (Parser.Exception e) {
			System.err.println("Parse error in file " + file);
			System.err.println(e.getMessage());
			System.exit(1);
		} /*catch (ScannerError e) {
			System.err.println("Scanner error in file " + file);
			System.err.println(e.getMessage());
			System.exit(1);
		}*/
		return null;
	}

	private void showUsageAndExit() {
		//System.out.println("Usage: java -jar bloqqi-compiler.jar [--option1] [--option2=value] ...  <filename1> [filename2] [filename3] ...");
		//System.exit(1);
	}

	public static void main(String[] args) {
		try {
			new Compiler(args);
		} catch (CommandLineException e) {
			System.out.println(e.getMessage());
			System.exit(1);
		}
	}
}

